#!/usr/bin/env python3
# Copyright 2022 The Wazo Authors  (see the AUTHORS file)
# SPDX-License-Identifier: GPL-3.0-or-later

import logging
import os
import sys

from wazo_auth_client import Client as AuthClient
from xivo.chain_map import ChainMap
from xivo.config_helper import read_config_file_hierarchy, parse_config_file

logger = logging.getLogger('fix_unassociated_all_users_policy')
logging.basicConfig(level=logging.INFO)

SENTINEL = '/var/lib/wazo-upgrade/fix-unassociated-all-users-policy'
_DEFAULT_CONFIG = {
    'config_file': '/etc/wazo-upgrade/config.yml',
    'auth': {
        'key_file': '/var/lib/wazo-auth-keys/wazo-upgrade-key.yml'
    }
}
ALL_USERS_POLICY_SLUG = 'wazo-enterprise-app-restrictions'


def load_config():
    file_config = read_config_file_hierarchy(_DEFAULT_CONFIG)
    key_config = _load_key_file(ChainMap(file_config, _DEFAULT_CONFIG))
    return ChainMap(key_config, file_config, _DEFAULT_CONFIG)


def _load_key_file(config):
    key_file = parse_config_file(config['auth']['key_file'])
    return {
        'auth': {
            'username': key_file['service_id'],
            'password': key_file['service_key'],
        },
    }


class AllUsersGroupNotFound(Exception):
    def __init__(self, group_name, tenant_uuid):
        super().__init__(f'{group_name}: no such group in tenant {tenant_uuid}')


def list_all_user_policies(auth_client):
    return auth_client.policies.list(recurse=True, slug=ALL_USERS_POLICY_SLUG)['items']


def associate_all_user_policy(auth_client, policy):
    tenant_uuid = policy['tenant_uuid']
    group_slug = f'wazo-all-users-tenant-{tenant_uuid}'
    groups = auth_client.groups.list(slug=group_slug, tenant_uuid=tenant_uuid)['items']
    try:
        group = groups[0]
    except IndexError:
        raise AllUsersGroupNotFound(group_slug, tenant_uuid)

    auth_client.groups.add_policy(group['uuid'], policy['uuid'], tenant_uuid=tenant_uuid)


def main():
    if os.path.exists(SENTINEL):
        sys.exit(0)

    config = load_config()
    auth_client = AuthClient(**config['auth'])
    token_data = auth_client.token.new(expiration=900)
    auth_client.set_token(token_data['token'])

    all_user_policies = list_all_user_policies(auth_client)
    logger.info('Found %s policies...', len(all_user_policies))
    for all_user_policy in all_user_policies:
        policy_uuid = all_user_policy['uuid']
        tenant_uuid = all_user_policy['tenant_uuid']
        logger.info('Tenant %s: associating policy %s to all users', tenant_uuid, policy_uuid)
        associate_all_user_policy(auth_client, all_user_policy)

    with open(SENTINEL, 'w') as f:
        f.write('')


if __name__ == '__main__':
    main()
